//
//	statevar: test the state variable classes
//

#include <cassert>
#include "../src/StateVariable.h"

enum Colors { RED, BLUE, YELLOW };
std::ostream &operator<<(std::ostream &os, Colors c)
{
	switch(c) {
		case RED: os<<"RED"; break;
		case BLUE: os<<"BLUE"; break;
		case YELLOW: os<<"YELLOW"; break;
		default: break;
	}

	return os;
}

static void checkE(double E)
{
	std::cout<<"Value of state variable E changed to : "<<E<<" eV"
			 <<std::endl;
}

int main()
{
	//
	//	a double range variable
	//
	RangeStateVariable<double> e(1200., "E", "eV", "energy",
								 true, true, 0., 15000.);
	e.notify(checkE);

	std::cout<<e<<std::endl;
	assert(e.get() == 1200.);
	e.set(1111.);
	std::cout<<e<<std::endl;
	assert(e.get() == 1111.);
	e.set(-2.);	
	assert(e.get() == 1111.);

	//
	//	a choice variable (bool)
	//
	vector<bool> bv; bv.push_back(false); bv.push_back(true);
	ChoiceStateVariable<bool> sw(false, "SW", "none", "boolean switch",
								 bv);

	assert(sw.get() == false);
	std::cout<<sw<<std::endl;
	sw.set(true);
	assert(sw.get() == true);
	std::cout<<sw<<std::endl;

	//
	//	another choice variable
	//
	vector<Colors> vc; vc.push_back(RED); vc.push_back(BLUE);
	ChoiceStateVariable<Colors> col(RED, "col", "color", "a color", vc);

	std::cout<<col<<std::endl;
	assert(col.get() == RED);
	col.set(YELLOW);	//	illegal value
	assert(col.get() == RED);
	col.set(BLUE);
	assert(col.get() == BLUE);
	std::cout<<col<<std::endl;

	return 0;
}
